# Databricks notebook source
# MAGIC %md
# MAGIC ## Read, transform, and write data
# MAGIC 
# MAGIC To work with data, you can use *Spark DataFrames*.

# COMMAND ----------

# reading Parquet data from mounted storage

# longer version
#df = spark.read.format('parquet').load('/mnt/ads-staging/Daily/parquet');

# shorter version
df = spark.read.parquet('/mnt/ads-staging/Daily/parquet');

# COMMAND ----------

# reading Json data from mounted storage

# longer version
#df = spark.read.format('json').load('/mnt/ads-globaldata/Demography');

#shortest version
df = spark.read.json('/mnt/ads-globaldata/Daily');

# COMMAND ----------

# 1. inferred schema
df = spark.read.json('/mnt/ads-globaldata/Daily');

# COMMAND ----------

# 2. explicit schema
from pyspark.sql.types import *;

fileSchema = StructType([
  StructField("Code", StringType()),
  StructField("Country", StringType()),
  StructField("date", StringType()),
  StructField("new_cases", LongType()),
  StructField("new_cases_per_million", DoubleType()),
  StructField("new_cases_smoothed", DoubleType()),
  StructField("new_cases_smoothed_per_million", DoubleType()),
  StructField("new_deaths", LongType()),
  StructField("new_deaths_per_million", DoubleType()),
  StructField("new_deaths_smoothed", DoubleType()),
  StructField("new_deaths_smoothed_per_million", DoubleType()),
  StructField("stringency_index", DoubleType()),
  StructField("total_cases", LongType()),
  StructField("total_cases_per_million", DoubleType()),
  StructField("total_deaths", LongType()),
  StructField("total_deaths_per_million", DoubleType())
]);

df = spark.read.json('/mnt/ads-globaldata/Daily', schema = fileSchema);

# COMMAND ----------

#display first 1000 record
display(df);

# COMMAND ----------

#display first 100 record
display(df.limit(100));

# COMMAND ----------

# MAGIC %fs ls /mnt/ads-globaldata/Demography

# COMMAND ----------

# reading all source files
dfData = spark.read.parquet('/mnt/ads-staging/Daily/parquet');
dfCountries = spark.read.json('/mnt/ads-globaldata/Demography');

# COMMAND ----------

display(dfCountries)

# COMMAND ----------

# creating temporary tables
dfData.createOrReplaceTempView('vwData');
dfCountries.createOrReplaceTempView('vwCountries');

# COMMAND ----------

# MAGIC %sql
# MAGIC 
# MAGIC /* testing the moving average */
# MAGIC SELECT
# MAGIC   d.date,
# MAGIC   AVG(d.new_cases) OVER (
# MAGIC     PARTITION BY d.Country 
# MAGIC     ORDER BY d.date 
# MAGIC     ROWS BETWEEN 6 PRECEDING AND CURRENT ROW) as new_cases_avg_last_7_days,
# MAGIC   c.continent
# MAGIC FROM vwData AS d
# MAGIC INNER JOIN vwCountries AS c  ON d.Country = c.CountryCode
# MAGIC WHERE
# MAGIC   c.Continent in ('North America', 'South America')

# COMMAND ----------

# performing the transformation
dfExtendedData = spark.sql('''
SELECT
  d.date,
  d.total_cases,
  d.new_cases,
  d.total_deaths,
  d.new_deaths,
  AVG(d.new_cases) OVER (
    PARTITION BY d.Country 
    ORDER BY d.date 
    ROWS BETWEEN 6 PRECEDING AND CURRENT ROW) as new_cases_avg_last_7_days,
  c.aged_65_older,
  c.aged_70_older,
  c.life_expectancy,
  c.population,
  c.population_density,
  c.location,
  c.continent
FROM vwData AS d
INNER JOIN vwCountries AS c  ON d.Country = c.CountryCode
WHERE
  c.Continent in ('North America', 'South America') 
''');

# COMMAND ----------

# writing to disk (engine decides number of partitions)
dfExtendedData.write.mode("overwrite").csv('/mnt/ads-staging/transform/covid19_data', header = True);

# COMMAND ----------

# writing to disk (fixed number of partitions)
dfExtendedData.coalesce(1).write.mode("overwrite").csv('/mnt/ads-staging/transform/covid19_data_single_file', header = True);

# COMMAND ----------

# MAGIC %fs ls /mnt/ads-staging/transform/covid19_data

# COMMAND ----------

# MAGIC %fs ls /mnt/ads-staging/transform/covid19_data_single_file
